package java.time.chrono;

import java.time.temporal.Temporal;
import java.io.DataInput;
import java.io.IOException;
import java.io.DataOutput;
import java.io.InvalidObjectException;
import java.io.ObjectInputStream;
import java.time.Period;
import java.time.LocalTime;
import java.time.temporal.TemporalUnit;
import java.time.temporal.TemporalAmount;
import java.time.temporal.TemporalAdjuster;
import java.time.temporal.UnsupportedTemporalTypeException;
import java.time.temporal.ChronoField;
import java.time.temporal.ValueRange;
import java.time.temporal.TemporalField;
import java.util.Objects;
import java.time.temporal.TemporalAccessor;
import java.time.ZoneId;
import java.time.Clock;
import java.time.LocalDate;
import java.io.Serializable;

public final class MinguoDate extends ChronoLocalDateImpl<MinguoDate> implements ChronoLocalDate, Serializable
{
    private static final long serialVersionUID = 1300372329181994526L;
    private final transient LocalDate isoDate;
    
    public static MinguoDate now() {
        return now(Clock.systemDefaultZone());
    }
    
    public static MinguoDate now(ZoneId zone) {
        return now(Clock.system(zone));
    }
    
    public static MinguoDate now(Clock clock) {
        return new MinguoDate(LocalDate.now(clock));
    }
    
    public static MinguoDate of(int n, int month, int dayOfMonth) {
        return new MinguoDate(LocalDate.of(n + 1911, month, dayOfMonth));
    }
    
    public static MinguoDate from(TemporalAccessor temporalAccessor) {
        return MinguoChronology.INSTANCE.date(temporalAccessor);
    }
    
    MinguoDate(LocalDate localDate) {
        Objects.requireNonNull(localDate, "isoDate");
        this.isoDate = localDate;
    }
    
    @Override
    public MinguoChronology getChronology() {
        return MinguoChronology.INSTANCE;
    }
    
    @Override
    public MinguoEra getEra() {
        return (this.getProlepticYear() >= 1) ? MinguoEra.ROC : MinguoEra.BEFORE_ROC;
    }
    
    @Override
    public int lengthOfMonth() {
        return this.isoDate.lengthOfMonth();
    }
    
    @Override
    public ValueRange range(TemporalField temporalField) {
        if ((temporalField instanceof ChronoField)) {
            if (this.isSupported(temporalField)) {
                ChronoField chronoField = (ChronoField)temporalField;
                switch (chronoField) {
                    case DAY_OF_MONTH:
                    case DAY_OF_YEAR:
                    case ALIGNED_WEEK_OF_MONTH: {
                        return this.isoDate.range(temporalField);
                    }
                    case YEAR_OF_ERA: {
                        ValueRange range = ChronoField.YEAR.range();
                        return ValueRange.of(1L, (this.getProlepticYear() <= 0) ? (-range.getMinimum() + 1L + 1911L) : (range.getMaximum() - 1911L));
                    }
                    default: {
                        return this.getChronology().range(chronoField);
                    }
                }
            }
            else {
                throw new UnsupportedTemporalTypeException("Unsupported field: " + temporalField);
            }
        }
        else {
            return temporalField.rangeRefinedBy(this);
        }
    }
    
    @Override
    public long getLong(TemporalField field) {
        if ((field instanceof ChronoField)) {
            switch ((ChronoField)field) {
                case PROLEPTIC_MONTH: {
                    return this.getProlepticMonth();
                }
                case YEAR_OF_ERA: {
                    int prolepticYear = this.getProlepticYear();
                    return (long)((prolepticYear >= 1) ? prolepticYear : (1 - prolepticYear));
                }
                case YEAR: {
                    return (long)this.getProlepticYear();
                }
                case ERA: {
                    return (long)(this.getProlepticYear() >= 1);
                }
                default: {
                    return this.isoDate.getLong(field);
                }
            }
        }
        else {
            return field.getFrom(this);
        }
    }
    
    private long getProlepticMonth() {
        return (long)this.getProlepticYear() * 12L + (long)this.isoDate.getMonthValue() - 1L;
    }
    
    private int getProlepticYear() {
        return this.isoDate.getYear() - 1911;
    }
    
    @Override
    public MinguoDate with(TemporalField field, long newValue) {
        if ((field instanceof ChronoField)) {
            ChronoField chronoField = (ChronoField)field;
            if (this.getLong(chronoField) == newValue) {
                return this;
            }
            switch (chronoField) {
                case PROLEPTIC_MONTH: {
                    this.getChronology().range(chronoField).checkValidValue(newValue, chronoField);
                    return this.plusMonths(newValue - this.getProlepticMonth());
                }
                case YEAR_OF_ERA:
                case YEAR:
                case ERA: {
                    int checkValidIntValue = this.getChronology().range(chronoField).checkValidIntValue(newValue, chronoField);
                    switch (chronoField) {
                        case YEAR_OF_ERA: {
                            return this.with(this.isoDate.withYear((this.getProlepticYear() >= 1) ? (checkValidIntValue + 1911) : (1 - checkValidIntValue + 1911)));
                        }
                        case YEAR: {
                            return this.with(this.isoDate.withYear(checkValidIntValue + 1911));
                        }
                        case ERA: {
                            return this.with(this.isoDate.withYear(1 - this.getProlepticYear() + 1911));
                        }
                        default: {
                            break;
                        }
                    }
                    break;
                }
            }
            return this.with(this.isoDate.with(field, newValue));
        }
        else {
            return (MinguoDate)super.with(field, newValue);
        }
    }
    
    @Override
    public MinguoDate with(TemporalAdjuster temporalAdjuster) {
        return (MinguoDate)super.with(temporalAdjuster);
    }
    
    @Override
    public MinguoDate plus(TemporalAmount temporalAmount) {
        return (MinguoDate)super.plus(temporalAmount);
    }
    
    @Override
    public MinguoDate minus(TemporalAmount temporalAmount) {
        return (MinguoDate)super.minus(temporalAmount);
    }
    
    @Override
    MinguoDate plusYears(long yearsToAdd) {
        return this.with(this.isoDate.plusYears(yearsToAdd));
    }
    
    @Override
    MinguoDate plusMonths(long monthsToAdd) {
        return this.with(this.isoDate.plusMonths(monthsToAdd));
    }
    
    @Override
    MinguoDate plusWeeks(long n) {
        return (MinguoDate)super.plusWeeks(n);
    }
    
    @Override
    MinguoDate plusDays(long daysToAdd) {
        return this.with(this.isoDate.plusDays(daysToAdd));
    }
    
    @Override
    public MinguoDate plus(long n, TemporalUnit temporalUnit) {
        return (MinguoDate)super.plus(n, temporalUnit);
    }
    
    @Override
    public MinguoDate minus(long n, TemporalUnit temporalUnit) {
        return (MinguoDate)super.minus(n, temporalUnit);
    }
    
    @Override
    MinguoDate minusYears(long n) {
        return (MinguoDate)super.minusYears(n);
    }
    
    @Override
    MinguoDate minusMonths(long n) {
        return (MinguoDate)super.minusMonths(n);
    }
    
    @Override
    MinguoDate minusWeeks(long n) {
        return (MinguoDate)super.minusWeeks(n);
    }
    
    @Override
    MinguoDate minusDays(long n) {
        return (MinguoDate)super.minusDays(n);
    }
    
    private MinguoDate with(LocalDate localDate) {
        return localDate.equals(this.isoDate) ? this : new MinguoDate(localDate);
    }
    
    @Override
    public final ChronoLocalDateTime<MinguoDate> atTime(LocalTime localTime) {
        return super.atTime(localTime);
    }
    
    @Override
    public ChronoPeriod until(ChronoLocalDate endDateExclusive) {
        Period until = this.isoDate.until(endDateExclusive);
        return this.getChronology().period(until.getYears(), until.getMonths(), until.getDays());
    }
    
    @Override
    public long toEpochDay() {
        return this.isoDate.toEpochDay();
    }
    
    @Override
    public boolean equals(Object o) {
        return this == o || (o instanceof MinguoDate && this.isoDate.equals(((MinguoDate)o).isoDate));
    }
    
    @Override
    public int hashCode() {
        return this.getChronology().getId().hashCode() ^ this.isoDate.hashCode();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws InvalidObjectException {
        throw new InvalidObjectException("Deserialization via serialization delegate");
    }
    
    private Object writeReplace() {
        return new Ser((byte)7, this);
    }
    
    void writeExternal(DataOutput dataOutput) throws IOException {
        dataOutput.writeInt(this.get(ChronoField.YEAR));
        dataOutput.writeByte(this.get(ChronoField.MONTH_OF_YEAR));
        dataOutput.writeByte(this.get(ChronoField.DAY_OF_MONTH));
    }
    
    static MinguoDate readExternal(DataInput dataInput) throws IOException {
        return MinguoChronology.INSTANCE.date(dataInput.readInt(), (int)dataInput.readByte(), (int)dataInput.readByte());
    }
}
